@@ -562,6 +562,7 @@ gst_audio_decoder_negotiate_default (GstAudioDecoder * dec)
   GstAudioDecoderClass *klass;
   gboolean res = TRUE;
   GstCaps *caps;
+  GstCaps *prevcaps;
   GstQuery *query = NULL;
   GstAllocator *allocator;
   GstAllocationParams params;
@@ -576,30 +577,32 @@ gst_audio_decoder_negotiate_default (GstAudioDecoder * dec)
   GST_DEBUG_OBJECT (dec, "setting src caps %" GST_PTR_FORMAT, caps);
 
   if (dec->priv->pending_events) {
-    GList *pending_events, *l;
-    gboolean set_caps = FALSE;
+    GList **pending_events, *l;
 
-    pending_events = dec->priv->pending_events;
-    dec->priv->pending_events = NULL;
+    pending_events = &dec->priv->pending_events;
 
     GST_DEBUG_OBJECT (dec, "Pushing pending events");
-    for (l = pending_events; l; l = l->next) {
+    for (l = *pending_events; l;) {
       GstEvent *event = GST_EVENT (l->data);
+      GList *tmp;
 
-      if (GST_EVENT_TYPE (event) > GST_EVENT_CAPS && !set_caps) {
-        res = gst_pad_set_caps (dec->srcpad, caps);
-        set_caps = TRUE;
+      if (GST_EVENT_TYPE (event) < GST_EVENT_CAPS) {
+        gst_audio_decoder_push_event (dec, l->data);
+        tmp = l;
+        l = l->next;
+        *pending_events = g_list_delete_link (*pending_events, tmp);
+      } else {
+        l = l->next;
       }
-      gst_audio_decoder_push_event (dec, l->data);
     }
-    g_list_free (pending_events);
-    if (!set_caps) {
-      res = gst_pad_set_caps (dec->srcpad, caps);
-    }
-  } else {
-    res = gst_pad_set_caps (dec->srcpad, caps);
   }
 
+  prevcaps = gst_pad_get_current_caps (dec->srcpad);
+  if (!prevcaps || !gst_caps_is_equal (prevcaps, caps))
+    res = gst_pad_set_caps (dec->srcpad, caps);
+  if (prevcaps)
+    gst_caps_unref (prevcaps);
+
   if (!res)
     goto done;
   dec->priv->ctx.output_format_changed = FALSE;
@@ -1701,6 +1704,14 @@ gst_audio_decoder_sink_eventfunc (GstAudioDecoder * dec, GstEvent * event)
     case GST_EVENT_STREAM_START:
       GST_AUDIO_DECODER_STREAM_LOCK (dec);
       gst_audio_decoder_drain (dec);
+
+      GST_DEBUG_OBJECT (dec, "received STREAM_START. Clearing taglist");
+      /* Flush our merged taglist after a STREAM_START */
+      if (dec->priv->taglist) {
+        gst_tag_list_unref (dec->priv->taglist);
+        dec->priv->taglist = NULL;
+      }
+      dec->priv->taglist_changed = FALSE;
       GST_AUDIO_DECODER_STREAM_UNLOCK (dec);
 
       ret = gst_audio_decoder_push_event (dec, event);
