@@ -279,6 +279,7 @@ static const FormatInfo formats[] = {
   {"application/x-subtitle-sami", N_("Sami subtitle format"), FLAG_SUB, ""},
   {"application/x-subtitle-tmplayer", N_("TMPlayer subtitle format"), FLAG_SUB,
       ""},
+  {"application/x-teletext", "Teletext", 0, ""},
   {"application/x-kate", "Kate", 0, ""},
   {"subtitle/x-kate", N_("Kate subtitle format"), FLAG_SUB, ""},
   {"subpicture/x-dvb", "DVB subtitles", FLAG_SUB, ""},
@@ -287,7 +288,6 @@ static const FormatInfo formats[] = {
   /* non-audio/video/container formats */
   {"hdv/aux-v", "HDV AUX-V", 0, ""},
   {"hdv/aux-a", "HDV AUX-A", 0, ""},
-  {"private/teletext", "Teletext", 0, ""},
 
   /* formats with dynamic descriptions */
   {"audio/mpeg", NULL, FLAG_AUDIO, ""},
@@ -840,7 +840,7 @@ gst_pb_utils_get_decoder_description (const GstCaps * caps)
 
   tmp = copy_and_clean_caps (caps);
 
-  g_return_val_if_fail (has_single_media_type (tmp), NULL);
+  g_return_val_if_fail (gst_caps_is_fixed (tmp), NULL);
 
   /* special-case RTP caps */
   if (caps_are_rtp_caps (tmp, "video", &str)) {
@@ -891,7 +891,7 @@ gst_pb_utils_get_encoder_description (const GstCaps * caps)
   g_return_val_if_fail (caps != NULL, NULL);
   g_return_val_if_fail (GST_IS_CAPS (caps), NULL);
   tmp = copy_and_clean_caps (caps);
-  g_return_val_if_fail (has_single_media_type (tmp), NULL);
+  g_return_val_if_fail (gst_caps_is_fixed (tmp), NULL);
 
   /* special-case RTP caps */
   if (caps_are_rtp_caps (tmp, "video", &str)) {
@@ -1022,7 +1022,7 @@ gst_pb_utils_get_codec_description (const GstCaps * caps)
   g_return_val_if_fail (caps != NULL, NULL);
   g_return_val_if_fail (GST_IS_CAPS (caps), NULL);
   tmp = copy_and_clean_caps (caps);
-  g_return_val_if_fail (has_single_media_type (tmp), NULL);
+  g_return_val_if_fail (gst_caps_is_fixed (tmp), NULL);
 
   info = find_format_info (tmp);
 
