@@ -194,6 +194,10 @@ gst_tag_register_tags_internal (gpointer unused)
       GST_TYPE_BUFFER, _("ID3v2 frame"), _("unparsed id3v2 tag frame"),
       gst_tag_merge_use_first);
 
+  gst_tag_register_static (GST_TAG_MUSICAL_KEY, GST_TAG_FLAG_META,
+      G_TYPE_STRING, _("musical-key"), _("Initial key in which the "
+          "sound starts"), gst_tag_merge_use_first);
+
   return NULL;
 }
 
